import pandas as pd
import numpy as np
from ...util.config import Configurator
from aip import AipNlp
from .keyword_parser import KeywordParser
import pkuseg
import json
import os
import pprint

class QaBot():
    def __init__(self):
        # Initialize Baidu NLP client SDK
        app_id = Configurator().config['bot']['baidu_aip']['app_id']
        api_key = Configurator().config['bot']['baidu_aip']['api_key']
        secret_key = Configurator().config['bot']['baidu_aip']['secret_key']
        self.baidu_client = AipNlp(app_id, api_key, secret_key)

        self.keyword_parser = KeywordParser()

        data_path = Configurator().config['bot']['data_path']

        # Initialize segmentation toolkit (pkuseg)
        self.seg = pkuseg.pkuseg(
            model_name = 'default', 
            user_dict = os.path.join(data_path, 'pkuseg_symbols.txt'),
            postag = True)

        self.keyword_type_dict = {}
        keywords = pd.read_csv(os.path.join(data_path, 'keywords.csv'))
        for keyword, msg_type in keywords.values:
            self.keyword_type_dict[keyword.strip()] = msg_type.strip()

        with open(os.path.join(data_path, 'fund_name.json'), 'r') as f:
            self.fund_info = json.load(f)
        
        with open(os.path.join(data_path, 'fund_inverted_index.json'), 'r') as f:
            self.inverted_index = json.load(f)
    
    def text_parser(self, text):
        # Default type is 'not_available'
        result = {
            'type': 'not_available',
            'data': {
                'message': text
            }
        }

        # doc: https://ai.baidu.com/ai-doc/NLP/zk6z52hds
        # sentiment: 表示情感极性分类结果，0:负向，1:中性，2:正向
        emotion = self.baidu_client.sentimentClassify(text)
        if 'items' in emotion:
            result['emotion'] = emotion['items']
        else:
            print(f'Baidu emotion analysis failed: {emotion}')

        word_tuples = self.seg.cut(text)

        # query fund (exact match)
        for tp in word_tuples:
            if tp[1] == 'n' and tp[0] in self.fund_info.keys():
                result['type'] = 'hqzx'
                result['data']['target'] = 'fund'
                result['data']['fund'] = self.fund_info[tp[0]]
                return result
        
        # query fund (fuzzy match)
        fm = self.fuzzy_match(word_tuples)
        if fm:
            values = list(fm.values())
            if len(values) > 0:
                result['type'] = 'hqzx'
                result['data']['target'] = 'fund'
                result['data']['fund'] = values[0]
                return result

        # keyword parse
        self.keyword_parser.parse(word_tuples, text, result)

         # If None is met above, try keyword matching
        for tp in word_tuples:
            if tp[0] in self.keyword_type_dict:
                result['type'] = self.keyword_type_dict[tp[0]]
                return result

        return result
    
    def fuzzy_match(self, word_tuples, threshold=3):
        phase = []
        index = 0
        length = len(word_tuples)
        while index < length:
            cur_idx = index
            tmp = []
            while 'n' in word_tuples[cur_idx][1] or 'v' in word_tuples[cur_idx][1] or 'j' in word_tuples[cur_idx][1] or 'a' in word_tuples[cur_idx][1]:
                tmp.append(word_tuples[cur_idx][0])
                cur_idx += 1
                if cur_idx >= length:
                    break

            if len(tmp) > 0:
                phase.append(tmp)
            index = cur_idx
            index += 1

        new_phase = []
        for key_ws in phase:
            length = len(key_ws)
            if length > 1:
                for i in range(1, length):
                    new_phase.append(key_ws[i:])
                for i in range(length, 0, -1):
                    new_phase.append(key_ws[:i])

        phase = phase + new_phase
        final_phase = [x for x in phase if len(x) > 1]

        records = []
        for key_ws in final_phase:
            key_len = len(key_ws)
            
            try:
                cur_dict = self.inverted_index[key_ws[0]].copy()
            except:
                continue
            
            for i in range(1, key_len):
                try:
                    tmp_dict = self.inverted_index[key_ws[i]].copy()
                except:
                    break
                
                removed_key = []
                for k in cur_dict.keys():
                    if k not in tmp_dict.keys():
                        removed_key.append(k)
                for k in removed_key:      
                    del cur_dict[k]
                        
                if len(cur_dict.keys()) <= 0:
                    break
            
            if len(cur_dict.keys()) <= 0 or i <= 1:
                continue
            
            records.append((cur_dict, len(cur_dict.keys())))
        
        if len(records) > 0:
            sr_records = sorted(records, key=lambda x: x[1])
            opt = sr_records[0][0]
            opt_num = sr_records[0][1]
            if opt_num > threshold:
                return None
            else:
                return opt
        else:
            return None


if __name__ == '__main__':
    qr = QaBot()
    while True:
        data = input('Q: ')
        result = qr.text_parser(data)
        pprint.pprint(result)




from typing import List, Dict
import json
import datetime
import pandas as pd
import numpy as np

from pandas.tseries.offsets import DateOffset

from .....util.calculator import Calculator
from ....manager.manager_fund import FundDataManager
from ....view.derived_models import ThirdPartyPortfolioIndicator, ThirdPartyPortfolioPositionLatest
from ....api.derived import DerivedDataApi


class ThirdPortfolioBase:

    _DIM_TYPE = {
        'daily_ret': DateOffset(days=1),
        'weekly_ret': DateOffset(weeks=1),
        'monthly_ret': DateOffset(months=1),
        'quarterly_ret': DateOffset(months=3),
        'half_yearly_ret': DateOffset(months=6),
        'yearly_ret': DateOffset(years=1),
    }

    def _calc_one_portfolio(self, x, fund_nav, po_pos, po_nav, end_date):
        try:
            pos_data = po_pos.loc[x.po_id, :]
        except KeyError:
            print(f'can not calc portfolio {x.po_id}')
            return
        last_pos = pd.Series(json.loads(pos_data.position))
        fund_nav = fund_nav[fund_nav.fund_id.isin(last_pos.index)].pivot(index='datetime', columns='fund_id', values='adjusted_net_value')
        fund_nav = fund_nav.sort_index()
        # 净值增长率
        rate = fund_nav.iloc[-1, :] / fund_nav.iloc[0, :]
        # 持仓权重根据净值增长率变化
        new_pos = last_pos * rate
        pos_sum = new_pos.sum()
        nav_rate = pos_sum / last_pos.sum()
        new_pos /= pos_sum
        # portfolio最新净值
        po_nav_df: pd.Series = po_nav[x.po_id]
        new_po_nav = po_nav_df.array[-1] * nav_rate
        po_nav_df.loc[end_date.isoformat()] = new_po_nav
        # print(po_nav_df)

        result = {'po_id': x.po_id, 'datetime': end_date.date(), 'po_src': x.po_src, 'position': new_pos.to_json(), 'nav': new_po_nav}
        result['annual_ret'] = np.exp(np.log(new_po_nav) / (po_nav_df.shape[0] / Calculator.TRADING_DAYS_PER_YEAR)) - 1
        mdd_part = po_nav_df[:] / po_nav_df[:].rolling(window=po_nav_df.shape[0], min_periods=1).max()
        result['mdd'] = 1 - mdd_part.min()
        if np.isnan(result['mdd']):
            result['mdd_date2'] = None
            result['mdd_date1'] = None
        else:
            result['mdd_date2'] = mdd_part.idxmin()
            result['mdd_date1'] = po_nav_df[:result['mdd_date2']].idxmax()
        result['from_setup_ret'] = new_po_nav - 1
        po_nav_df.index = pd.to_datetime(po_nav_df.index)
        resampled_w = po_nav_df.resample('1W').apply(lambda x: x.array[-1] if not x.empty else np.nan)
        result['vol_by_week'] = resampled_w.dropna().pct_change().std(ddof=1) * np.sqrt(52)

        resampled_m = po_nav_df.resample('1M').apply(lambda x: x.array[-1] if not x.empty else np.nan)
        result['vol_by_month'] = resampled_m.dropna().pct_change().std(ddof=1) * np.sqrt(12)
        result['vol_by_day'] = po_nav_df.dropna().pct_change().std(ddof=1) * np.sqrt(244)
        result['annual_compounded_ret'] = np.power(new_po_nav, 1 / (po_nav_df.shape[0] / 244)) - 1
        if result['vol_by_week'] != 0:
            result['sharpe_ratio'] = (result['annual_compounded_ret'] - 0.02) / result['vol_by_week']
        else:
            result['sharpe_ratio'] = np.nan

        # 遍历每个需要计算的时间区间
        for name, value in self._DIM_TYPE.items():
            bd = (result['datetime'] - value).date()
            # 试图寻找bd及其之前的最近一天
            filtered_mv = po_nav_df.loc[po_nav_df.index.date <= bd]
            if filtered_mv.empty:
                continue
            # 计算区间内的return
            result[name] = po_nav_df.array[-1] / filtered_mv.array[-1] - 1

        result['nav'] = new_po_nav
        return pd.Series(result)

    def calc_everyday(self, end_date: str, po_src_list: List[int]):
        from ....view.cas.third_party_portfolio_nav import ThirdPartyPortfolioNav

        end_date_dt = pd.to_datetime(end_date, infer_datetime_format=True)
        _trading_day: pd.DataFrame = FundDataManager.basic_data('get_trading_day_list', start_date=(end_date_dt - DateOffset(months=1)).isoformat(), end_date=end_date).drop(columns='_update_time')

        fund_nav = FundDataManager.basic_data('get_fund_nav_with_date', start_date=_trading_day.datetime.array[-2], end_date=_trading_day.datetime.array[-1])

        # 获取所有portfolio信息
        po_info = DerivedDataApi().get_market_portfolio_info().drop(columns='_update_time')
        po_info = po_info[po_info.po_src.isin(po_src_list)]
        # 获取所有portfolio最新持仓
        po_pos = DerivedDataApi().get_market_portfolio_position_by_src(po_src_list).drop(columns='_update_time')
        po_pos = po_pos.set_index('po_id')
        # 获取所有portfolio最新净值
        po_nav: Dict[str, pd.Series] = {}
        for one in ThirdPartyPortfolioNav.objects.filter(po_id__in=po_info.po_id.to_list()):
            po_nav[one.po_id] = pd.Series(one.nav, index=one.date)

        # 遍历每个portfolio计算
        result = po_info.apply(self._calc_one_portfolio, axis=1, fund_nav=fund_nav, po_pos=po_pos, po_nav=po_nav, end_date=end_date_dt)
        result = result[result.notna().any(axis=1)].set_index('po_id').replace({np.nan: None})
        for col in result:
            if result[col].dtype in ('float64', 'int64'):
                result[col] = result[col].astype('object')
            elif isinstance(result[col].array[0], datetime.date):
                result[col] = result[col].transform(lambda x: x.isoformat())
        print(result)

        from sqlalchemy.orm import sessionmaker
        from surfing.data.wrapper.mysql import DerivedDatabaseConnector

        print(f'to update data of {ThirdPartyPortfolioPositionLatest.__table__.name}')
        Session = sessionmaker(DerivedDatabaseConnector().get_engine())
        db_session = Session()
        for row in db_session.query(ThirdPartyPortfolioPositionLatest).filter(ThirdPartyPortfolioPositionLatest.po_src.in_(po_src_list)).all():
            try:
                row_data = result.loc[row.po_id, :]
                row.datetime = row_data.datetime
                row.position = row_data.position
            except KeyError:
                print(f'not found {row.po_id} data')
        print('done, to commit')
        db_session.commit()

        print(f'to update data of {ThirdPartyPortfolioIndicator.__table__.name}')
        for row in db_session.query(ThirdPartyPortfolioIndicator).filter(ThirdPartyPortfolioIndicator.po_src.in_(po_src_list)).all():
            try:
                row_data = result.loc[row.po_id, :]
                row.datetime = row_data.datetime
                row.nav = row_data.nav
                row.sharpe_ratio = row_data.sharpe_ratio
                row.mdd = row_data.mdd
                row.annual_compounded_ret = row_data.annual_compounded_ret
                row.daily_ret = row_data.daily_ret
                row.weekly_ret = row_data.weekly_ret
                row.monthly_ret = row_data.monthly_ret
                row.quarterly_ret = row_data.quarterly_ret
                row.half_yearly_ret = row_data.half_yearly_ret
                row.yearly_ret = row_data.yearly_ret
                row.from_setup_ret = row_data.from_setup_ret
                row.vol_by_day = row_data.vol_by_day
                row.vol_by_week = row_data.vol_by_week
                row.vol_by_month = row_data.vol_by_month
                row.annual_ret = row_data.annual_ret
                row.mdd_d1 = row_data.mdd_date1
                row.mdd_d2 = row_data.mdd_date2
            except KeyError:
                print(f'not found {row.po_id} data')
        print('done, to commit')
        db_session.commit()
        db_session.close()

        for row in result.itertuples():
            ThirdPartyPortfolioNav.objects.filter(po_id=row.Index).update(
                date__append=[row.datetime],
                nav__append=[row.nav],
            )




from typing import List, Dict
import pandas as pd
import numpy as np
import math
import datetime
import traceback
import json
import statsmodels.api as sm
from multiprocessing import Pool
from scipy.optimize import Bounds, minimize
from statsmodels.tsa.ar_model import AutoReg
from ...api.basic import BasicDataApi
from ...api.raw import RawDataApi
from ...view.derived_models import FundAlpha
from .derived_data_helper import DerivedDataHelper


class FundAlphaProcessor:

    _TRADING_DAYS_PER_YEAR = 242
    _NATURAL_DAYS_PER_YEAR = 365
    _RISK_FEE_RATE = 0.011
    _RISK_FEE_RATE_PER_DAY = _RISK_FEE_RATE / _TRADING_DAYS_PER_YEAR
    _REPORT_DATE_LIST = ['0331', '0630', '0930', '1231']
    FUND_CLASSIFIER = {
        'QDII': ['国际(QDII)股票型基金', '国际(QDII)债券型基金', '国际(QDII)另类投资基金', '国际(QDII)混合型基金', ],
        'stock': ['普通股票型基金', '偏股混合型基金',  '被动指数型基金', '增强指数型基金', '平衡混合型基金', '灵活配置型基金', '股票多空'], 
        'bond': ['中长期纯债型基金', '短期纯债型基金', '偏债混合型基金', '增强指数型债券基金', '被动指数型债券基金','混合债券型二级基金', '混合债券型一级基金'],
        'mmf': ['货币市场型基金'],
        'index': ['被动指数型基金', '增强指数型基金', '增强指数型债券基金', '被动指数型债券基金', '商品型基金', 'REITs'],
    }
    REPLACE_DICT = {'stock': 'csi_stockfund',
                     'bond': 'csi_boodfund', 
                     'mmf':  'mmf', 
                     'QDII': 'csi_f_qdii', 
                     'index':'hs300'}
    _W1 = 5
    _M1 = 20
    _M3 = _M1 * 3
    _M6 = _M1 * 6
    _Y1 = _TRADING_DAYS_PER_YEAR
    _Y3 = _Y1 * 3
    _Y5 = _Y1 * 5
    _Y10 = _Y1 * 10
    
    def __init__(self, data_helper: DerivedDataHelper):
        self._data_helper = data_helper
        self._basic_api = BasicDataApi()
        
    def init(self, start_date: str, end_date: str):
        self.wind_type_dict = {}
        for type_i, type_list in self.FUND_CLASSIFIER.items():
            for _ in type_list:
                self.wind_type_dict.update({_:type_i})
        # 向前取12年
        start_date: str = (pd.to_datetime(start_date, infer_datetime_format=True) - datetime.timedelta(days=self._NATURAL_DAYS_PER_YEAR*15+1)).strftime('%Y%m%d')
        print(f'start date for data: {start_date}')
        # 获取区间内交易日列表
        all_trading_days: pd.Series = self._basic_api.get_trading_day_list().drop(columns='_update_time').set_index('datetime')
        self._trading_days: pd.Series = all_trading_days.loc[pd.to_datetime(start_date, infer_datetime_format=True).date():pd.to_datetime(end_date, infer_datetime_format=True).date()]
        # 使用实际trading day的最后一天作为end_date
        self.end_date = self._trading_days.index.array[-1]
        self.begin_of_year = datetime.date(self.end_date.year,1,1)
        self.this_y_lenth = self._trading_days.loc[self.begin_of_year:].shape[0]
        self.fund_info = self._basic_api.get_fund_info().drop(columns='_update_time')
        self._wind_class_2_dict = self.fund_info.set_index('fund_id')['wind_class_2'].to_dict()
        self.fund_info = self.fund_info[(self.fund_info.end_date >= self.end_date) 
                                       & (self.fund_info.structure_type <= 1)
                                       & (~self.fund_info.wind_class_2.isnull())]
        # 基金净值数据、指数价格数据的index对齐到交易日列表
        fund_nav: pd.DataFrame = self._basic_api.get_fund_nav_with_date(start_date, end_date)  #, fund_list = f_l)
        self._fund_nav: pd.DataFrame = fund_nav.pivot_table(index='datetime', columns='fund_id', values='adjusted_net_value').reindex(self._trading_days.index).fillna(method='ffill')
        # 排除掉异常净值基金
        res = []
        for fund_id in self._fund_nav:
            _df = self._fund_nav[fund_id].dropna()
            if _df.pct_change(1).abs().max() < 0.2:
                res.append(fund_id)
        self._fund_nav = self._fund_nav[res].copy()

        # 宏观数据
        self._macroeco = RawDataApi().get_em_macroeconomic_daily(codes=['EX_DR_RATIO']).drop(columns='_update_time')
        self._macroeco = self._macroeco.pivot_table(index='datetime', columns='codes', values='value')
        self._macroeco = self._macroeco.reindex(self._trading_days.index.union(self._macroeco.index)).fillna(method='ffill').reindex(self._trading_days.index)

        index_price: pd.DataFrame = self._basic_api.get_index_price_dt(start_date, end_date).drop(columns='_update_time')
        # 有的index最近没有数据，如活期存款利率/各种定期存款利率等，需要先reindex到全部的交易日列表上ffill
        self._index_price: pd.DataFrame = index_price.pivot_table(index='datetime', columns='index_id', values='close').reindex(self._trading_days.index).fillna(method='ffill')
        _bank_rate_df: pd.DataFrame = self._basic_api.get_index_price(index_list=['ddir', 'nonor', 'tmd_1y', 'tmd_2y', 'tmd_3m', 'tmd_3y', 'tmd_5y', 'tmd_6m', 'tmd_7d']).drop(columns='_update_time')
        _bank_rate_df = _bank_rate_df.pivot_table(index='datetime', columns='index_id', values='close').reindex(all_trading_days.index).fillna(method='ffill').reindex(self._trading_days.index)
        drop_list = [i for i in self._index_price if i in _bank_rate_df]
        self._index_price = self._index_price.drop(drop_list, axis=1)
        self._index_price: pd.DataFrame = pd.concat([self._index_price, _bank_rate_df], axis=1)
        # 再reindex到参与计算的交易日列表上
        # self._index_price: pd.DataFrame = index_price.reindex(self._trading_days.index)
        pd.testing.assert_index_equal(self._fund_nav.index, self._index_price.index)
        try:
            # 这个指数有一天的点数是0，特殊处理一下
            self._index_price['spi_spa'] = self._index_price['spi_spa'].where(self._index_price.spi_spa != 0).fillna(method='ffill')
        except KeyError:
            pass

        # 对数净值取差分并且去掉第一个空值，得到基金对数收益率数列
        self._fund_ret: pd.DataFrame = np.log(self._fund_nav).diff().iloc[1:, :]

        # 计算benchmark return，且index对齐到fund_ret
        self.fund_benchmark_df = self._basic_api.get_fund_benchmark().drop(columns='_update_time')
        benchmark_ret: pd.DataFrame = self._get_benchmark_return()
        self._benchmark_ret: pd.DataFrame = benchmark_ret.reindex(self._fund_ret.index)
        self._benchmark_ret = self._benchmark_ret.reindex(self._trading_days.index)
        benchmark_cols = self._benchmark_ret.columns.tolist()
        for fund_id in self._fund_nav.columns:
            benchmark_id = fund_id
            if not benchmark_id in benchmark_cols:    
                wind_class_2 = self._wind_class_2_dict[fund_id]
                if wind_class_2 not in self.wind_type_dict.keys():
                    continue
                select_type = self.wind_type_dict[wind_class_2]
                benchmark_ret = self._index_price[self.REPLACE_DICT[select_type]].pct_change(1).rename(benchmark_id)
                self._benchmark_ret = self._benchmark_ret.join(benchmark_ret)
        self._benchmark_ret = self._benchmark_ret.reindex(self._fund_ret.index).fillna(method='ffill')
        pd.testing.assert_index_equal(self._fund_ret.index, self._benchmark_ret.index)

        # 获取待计算的基金列表
        fund_list: pd.DataFrame = self._basic_api.get_fund_info().drop(columns='_update_time')
        self._fund_list: pd.DataFrame = fund_list[fund_list.structure_type <= 1]
        
        self._hs300 = self._basic_api.get_index_price(tuple(['hs300']))[['datetime','close']].set_index('datetime').rename(columns={'close':'benchmark'})
        self._hs300 = np.log(self._hs300).diff().iloc[1:, :]
        
    def _get_benchmark_return(self) -> pd.DataFrame:
        benchmark_list: Dict[str, float] = {}
        # 遍历每一只基金的benchmark进行处理
        for row in self.fund_benchmark_df.itertuples(index=False):
            values: List[pd.Series] = []
            cons: float = 0
            # 空的benchmark表明我们没有对应的指数或无法解析公式
            if row.benchmark_s:
                benchmark: Dict[str, float] = json.loads(row.benchmark_s)
                benchmark_raw: Dict[str, float] = eval(row.benchmark)
                for (index, weight), index_raw in zip(benchmark.items(), benchmark_raw.keys()):
                    if index == '1':
                        # 表示benchmark中该项为常数
                        cons += weight
                    elif index in ('ddir', 'nonor', 'tmd_1y', 'tmd_2y', 'tmd_3m', 'tmd_3y', 'tmd_5y', 'tmd_6m', 'tmd_7d'):
                        if weight == -1:
                            # 表示我们无法解析公式
                            # print(f'[benchmark_return] Error: Need fix {row.fund_id} {index} {index_raw}')
                            # self.log.append((row.fund_id, index))
                            break
                        else:
                            try:
                                if index == 'nonor':
                                    # 在这里我们用超额存款准备金率替代同业存款利率
                                    ra: pd.Series = self._macroeco.loc[:, 'EX_DR_RATIO']
                                else:
                                    ra: pd.Series = self._index_price.loc[:, index]
                            except KeyError:
                                # 表示我们没有该指数的价格数据
                                # print(f'[benchmark_return] Error: Data Missing: {row.fund_id} {index} {index_raw}')
                                # self.log.append((row.fund_id, index))
                                break
                            else:
                                values.append(ra.iloc[1:] * 0.01 * weight / self._TRADING_DAYS_PER_YEAR)
                    else:
                        if weight == -1:
                            # 表示我们无法解析公式
                            # print(f'[benchmark_return] Error: Need fix {row.fund_id} {index} {index_raw}')
                            # self.log.append((row.fund_id, index))
                            break
                        else:
                            try:
                                ra: pd.Series = self._index_price.loc[:, index]
                            except KeyError:
                                # 表示我们没有该指数的价格数据
                                # print(f'Error: Data Missing: {row.fund_id} {index} {index_raw}')
                                # self.log.append((row.fund_id, index))
                                break
                            else:
                                ra = np.log(ra).diff().iloc[1:]
                                values.append(ra * weight)
                else:
                    if values or cons:
                        the_sum: float = sum(values)
                        if cons:
                            the_sum += np.log(math.pow(1 + cons, 1 / self._TRADING_DAYS_PER_YEAR))
                        benchmark_list[row.fund_id] = the_sum

        return pd.DataFrame.from_dict(benchmark_list)

    def get_alpha(self, df, term):
        term = min(df.shape[0], term) 
        ploy_res = np.polyfit(y=df.tail(term).fund, x=df.tail(term).benchmark, deg=1)
        alpha = ploy_res[1] * term
        return alpha
    
    def get_track_err(self, df):
        return (df.fund - df.benchmark).std(ddof=1) * np.sqrt(self._TRADING_DAYS_PER_YEAR)

    def calculate_item(self, fund_id):
        try:
            fund_ret_i = self._fund_ret[[fund_id]].rename(columns={fund_id:'fund'})
            if fund_id in self._benchmark_ret:
                bench_ret_i = self._benchmark_ret[[fund_id]].rename(columns={fund_id:'benchmark'})
            else:
                bench_ret_i = self._hs300
            df_i = fund_ret_i.join(bench_ret_i).dropna()
            
            track_err = self.get_track_err(df_i)
            this_y_alpha = self.get_alpha(df_i, self.this_y_lenth)
            cumulative_alpha = self.get_alpha(df_i, df_i.shape[0])
            w1_alpha = self.get_alpha(df_i, self._W1)
            m1_alpha = self.get_alpha(df_i, self._M1)
            m3_alpha = self.get_alpha(df_i, self._M3)
            m6_alpha = self.get_alpha(df_i, self._M6)
            y1_alpha = self.get_alpha(df_i, self._Y1)
            y3_alpha = self.get_alpha(df_i, self._Y3)
            y5_alpha = self.get_alpha(df_i, self._Y5)
            y10_alpha = self.get_alpha(df_i, self._Y10)

            dic = {
                'track_err' : track_err,
                'this_y_alpha' : this_y_alpha,
                'cumulative_alpha' : cumulative_alpha,
                'w1_alpha' : w1_alpha,
                'm1_alpha' : m1_alpha,
                'm3_alpha' : m3_alpha,
                'm6_alpha' : m6_alpha,
                'y1_alpha' : y1_alpha,
                'y3_alpha' : y3_alpha,
                'y5_alpha' : y5_alpha,
                'y10_alpha' : y10_alpha,
                'fund_id' : fund_id,
                'datetime' : self.end_date
            }
            return dic
        except Exception:
            print('fund_id can not calculate alpha', fund_id)
            return None

    def calculate(self):
        fund_list = self._fund_ret.columns.to_list()
        p = Pool()
        res = [i for i in p.imap_unordered(self.calculate_item, fund_list, 256) if i is not None]
        p.close()
        p.join()
        self.result = pd.DataFrame(res)

    def process(self, start_date, end_date):
        failed_tasks = []
        try:
            self.init(start_date, end_date)
            self.calculate()
            self._data_helper._upload_derived(self.result, FundAlpha.__table__.name)
        except Exception as e:
            print(e)
            traceback.print_exc()
            failed_tasks.append(FundAlpha.__table__.name)
        return failed_tasks


if __name__ == '__main__':
    fap = FundAlphaProcessor(DerivedDataHelper())
    fap.process('20200925', '20200925')



import time
import json
import datetime
from typing import Union, Tuple

import numpy as np
import statsmodels.api as sm
import pandas as pd
from sklearn.preprocessing import StandardScaler, MinMaxScaler

from collections import defaultdict
from ...wrapper.mysql import DerivedDatabaseConnector

FUND_CLASSIFIER = {
    'stock': ['偏股混合型基金', '普通股票型基金', '增强指数型基金', '被动指数型基金'],
    'bond': ['混合债券型二级基金', '中长期纯债型基金', '增强指数型债券基金', '混合债券型一级基金', '被动指数型债券基金', '短期纯债型基金', '偏债混合型基金'],
    'QDII': ['国际(QDII)另类投资基金', '国际(QDII)混合型基金', '国际(QDII)股票型基金', '国际(QDII)债券型基金', '股票多空', '商品型基金', 'REITs'],
    'mmf': ['货币市场型基金'],
    'mix': ['平衡混合型基金', '灵活配置型基金']
}

class DateJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.date):
            return obj.isoformat()
        # Let the base class default method raise the TypeError
        return json.JSONEncoder.default(self, obj)


def normalize(a: Union[pd.DataFrame, pd.Series]) -> np.ndarray:
    if isinstance(a, pd.Series):
        a = pd.DataFrame(a)
    return StandardScaler().fit_transform(a)

def score_rescale(a: Union[pd.DataFrame, pd.Series, np.ndarray]) -> np.ndarray:
    if isinstance(a, pd.Series) or isinstance(a, np.ndarray):
        a = pd.DataFrame(a)
    a = a.rank()
    return MinMaxScaler().fit_transform(a) * 100

def cont_rescale(a: Union[pd.DataFrame, pd.Series], feature_range: Tuple[float, float] = (0, 1)) -> np.ndarray:
    if isinstance(a, pd.Series):
        a = pd.DataFrame(a)
    return MinMaxScaler(feature_range=feature_range).fit_transform(a)


class DerivedDataHelper:
    def __init__(self):
        self._updated_count = defaultdict(int)
        self._timer = time.monotonic()

    @staticmethod
    def _lambda_cl(total: np.ndarray):
        if total.shape[0] <= 1:
            return {'beta':np.nan,
                    'alpha':np.nan,
                    'alpha_t':np.nan,
                    'alpha_p':np.nan,
                    'beta_t':np.nan,
                    'beta_p':np.nan,
                    }
        X = np.array([total[:, 1], total[:, 1]]).T
        X[:, 1][X[:, 1] > 0] = 0
        if np.count_nonzero(X[:, 1]) == 0:
            return {'beta':np.nan,
                    'alpha':np.nan,
                    'alpha_t':np.nan,
                    'alpha_p':np.nan,
                    'beta_t':np.nan,
                    'beta_p':np.nan,
                    }
        X[:, 0][X[:, 0] < 0] = 0
        est2 = sm.OLS(total[:, 0], sm.add_constant(X, prepend=False)).fit()
        return {'beta':est2.params[0] - est2.params[1],
                'alpha':est2.params[-1],
                'alpha_t':est2.tvalues[-1],
                'alpha_p':est2.pvalues[-1],
                # TODO beta 2 - beta 1 , p values and t values calculation is not correct
                'beta_t':est2.tvalues[1],
                'beta_p':est2.pvalues[1]
                }

    def _upload_derived(self, df, table_name, to_truncate=False):
        print(table_name)
        # print(df)
        if to_truncate:
            with DerivedDatabaseConnector().managed_session() as mn_session:
                try:
                    mn_session.execute(f'TRUNCATE TABLE {table_name}')
                    mn_session.commit()
                except Exception as e:
                    print(f'Failed to truncate table {table_name} <err_msg> {e}')

        df = df.drop(columns='_update_time', errors='ignore')
        df.to_sql(table_name, DerivedDatabaseConnector().get_engine(), index=False, if_exists='append')

        now: float = time.monotonic()
        print(f'{table_name} costs {now - self._timer}s')
        self._timer = now
        self._updated_count[table_name] += df.shape[0]



import datetime
import pandas as pd
import numpy as np
import scipy.optimize as sco
from numpy.linalg import inv

class ModelCalulator:

    STATS_DICT = {
        'ret':0,
        'std':1,
        'sharpe':2,
    }

    def __init__(self, risk_free:float=0.03, trading_day_per_year:int=242):
        """
        - ''risk_free'' - (float) risk free rate default 0.03
        - ''trading_day_per_year'' -(int) trading day each year 242 or 252
        """
        self.risk_free = risk_free
        self.trading_day_per_year = trading_day_per_year

    def _calc_statistics(self, weights, *args):    
        returns = args[0]
        if len(args) == 1:
            ret_mean = returns.mean()
            ret_cov = returns.cov()
        else:
            ret_mean = args[1]
            ret_cov = args[2]
            
        weights = list(weights)
        
        weights = np.array(weights)

        port_ret = np.sum(ret_mean * weights)*self.trading_day_per_year

        port_std = np.sqrt(np.dot(weights.T, np.dot(ret_cov*self.trading_day_per_year,weights)))

        return np.array([port_ret, port_std, (port_ret-self.risk_free)/port_std])

    def _cons_parse(self, returns:pd.DataFrame, asset_weight_cons:list=[], port_stats_cons:list=[]):
        asset_list = returns.columns.tolist() 
        # 默认权重合为1
        cons = ({'type':'eq','fun':lambda x: np.sum(x) - 1},)
        # 资产权重条件
        for asset_weight_con_i in asset_weight_cons:
            asset_id = asset_weight_con_i[0]
            method = asset_weight_con_i[1]
            num = asset_weight_con_i[2]
            asset_idx = asset_list.index(asset_id)
            print(asset_id, method, num, asset_idx)
            if method == '=':
                cons += ({'type':'eq','fun':lambda x: x[asset_idx] - num},)
            elif method == '>=':
                cons += ({'type':'ineq','fun':lambda x: x[asset_idx] - num},) 
            elif method == '<=':
                cons += ({'type':'ineq','fun':lambda x: -x[asset_idx] + num},) 
        # 组合指标条件
        for port_stats_con_i in port_stats_cons:
            stat_name = port_stats_con_i[0]
            method = port_stats_con_i[1]
            num = port_stats_con_i[2]
            idx = self.STATS_DICT[stat_name]
            print(stat_name, method, num, idx)
            if method == '=':
                cons += ({'type':'eq','fun':lambda x: self._calc_statistics(x,(returns))[idx] - num}, )
            elif method == '>=':
                cons += ({'type':'ineq','fun':lambda x: self._calc_statistics(x,(returns))[idx] - num}, )
            elif method == '<=':
                cons += ({'type':'ineq','fun':lambda x: -self._calc_statistics(x,(returns))[idx] + num}, )
        return cons

    def _view_parse(self, investor_views:list, returns:pd.DataFrame, market_weights:list):
        def _asset_weight_sum(assets, market_weights, asset_list):
            return sum(market_weights[asset_list.index(asset_id)] for asset_id in assets)
        
        asset_list = returns.columns.tolist()
        views = []
        pick_list = []
        view_confidences = []
        for view_i in investor_views:
            good_assets = view_i[0]
            bad_assets = view_i[1]
            ret = view_i[2]
            confidence = view_i[3]
            views.append(ret)
            view_confidences.append(confidence)
            pick_dict = {}
            good_asset_weights = _asset_weight_sum(good_assets, market_weights, asset_list)
            bad_asset_weights = _asset_weight_sum(bad_assets, market_weights, asset_list)
            for asset_id in good_assets:
                w = market_weights[asset_list.index(asset_id)]
                pick_dict[asset_id] = w / good_asset_weights
            for asset_id in bad_assets:
                w = market_weights[asset_list.index(asset_id)]
                pick_dict[asset_id] = - w/ bad_asset_weights
            pick_list.append(pick_dict)
        return views, pick_list, view_confidences    

    def _max_sharpe(self, weights, *args):   
        return -self._calc_statistics(weights, *args)[2]

    def _min_vol(self, weights, *args):
        return self._calc_statistics(weights, *args)[1]
        
    def _max_ret(self, weights, *args):
        return -self._calc_statistics(weights, *args)[0]

    def _get_opt_weight(self, opts, fund_list):
        return pd.DataFrame(np.reshape(opts.x.round(3), newshape=(1,len(opts.x))), columns=fund_list)

    def _get_port_stats(self, opts, returns):
        return pd.DataFrame(np.reshape(self._calc_statistics(opts.x,(returns)).round(3), newshape=(1,3)),columns=['年化收益','年化波动','夏普率'])

    def _calculate_risk(self, risk_method, returns, fund_nav, day_per_year):
        if risk_method == 'annual_vol':
            risk = returns.std(ddof=1)*np.sqrt(day_per_year)
        elif risk_method == 'mdd':
            risk = 1 - (fund_nav / fund_nav.cummax()).min()
        elif risk_method == 'var':
            risk = returns.quantile(0.05)
        elif risk_method == 'cvar':
            var = returns.quantile(0.05)
            risk = returns[returns < var].mean()
        risk = np.array(risk)
        return risk
        
    def _risk_budget_objective(self, x, *pars):
        risk_method = pars[0]           
        target_risk_weight = pars[1]         
        returns = pars[2] 
        fund_nav = pars[3]    
        day_per_year = pars[4]

        risk = self._calculate_risk_contribution(x, risk_method, returns, fund_nav, day_per_year)
        risk = risk * 100
        s2 = np.sum(target_risk_weight)
        target_risk_weight = target_risk_weight/s2*100
        s = np.square(risk-target_risk_weight.T)
        J = np.sum(np.diagonal(s))
        return J

    def _calculate_risk_contribution(self, x, risk_method, returns, fund_nav, day_per_year):
        risk = self._calculate_risk(risk_method, returns, fund_nav, day_per_year)
        risk = risk * np.array(x)
        risk.resize((risk.shape[0],1))
        risk = np.matrix(risk)
        s1 = np.sum(risk)
        risk = risk/s1
        return risk

    def _total_weight_constraint(self,x):
        return np.sum(x)-1.0

    def _long_only_constraint(self,x):
        return x

    def get_opt_weight(self):
        return self.weights
    
    def get_ports_stats(self):
        return self.port_stats

    def get_asset_risk(self):
        return self.port_asset_risk


class MeanVarianceModel(ModelCalulator):

    def __init__(self,risk_free:float=0.03, trading_day_per_year:int=242): 
        super().__init__(risk_free=risk_free, trading_day_per_year=trading_day_per_year)
        
    def process(self, returns:pd.DataFrame,opt_method:str,asset_weight_cons:list=[],port_stats_cons:list=[]):
        """
        均值方差模型

        - ''returns'' - (pd.DataFrame) columns name is asset name
        - ''opt_method'' str,  in ['max_sharpe', 'limit_std', 'limit_ret']
        - ''asset_weight_cons'' - (list of list) 某资产权重限制 
            每单元由三部分组成： [000001!0, '<=', 0.6]
                        1 资产id （str） 
                        2 大小方式（str） ’=’ ‘>=’ '<='
                        3 权重数  (float)    
        - ''port_stats_cons'' - (list of list) 组合指标限制
            每单元由三部分组成：['std', '<=', 0.2]
                        1 指标：['ret','std','sharpe']
                        2 大小方式（str） ’=’ ‘>=’ '<='
                        3 指标数  (float)                      
        """ 
        cons = self._cons_parse(returns, asset_weight_cons, port_stats_cons)
        # 增加资产cash
        returns.loc[:,'cash'] = 0
        asset_list = returns.columns.tolist()
        noa = len(asset_list)
        bnds = tuple((0,1) for x in range(noa)) # 权重限制在0,1之间 不允许做空
        if opt_method == 'max_sharpe':
            opt_func = self._max_sharpe
        elif opt_method == 'max_ret':
            opt_func = self._max_ret
        elif opt_method == 'min_std':
            opt_func = self._min_vol
        opts = sco.minimize(fun=opt_func, args=(returns), x0=noa*[1./noa,], method='SLSQP', bounds=bnds, constraints=cons)
        self.weights = self._get_opt_weight(opts, asset_list)
        self.port_stats = self._get_port_stats(opts, returns)

class BlackLittermanModel(ModelCalulator):

    def __init__(self,risk_free:float=0.03, trading_day_per_year:int=242): 
        super().__init__(risk_free=risk_free, trading_day_per_year=trading_day_per_year)

    def process(self,   returns:pd.DataFrame,
                        market_weights:list,
                        investor_views:list,
                        tau:float,
                        opt_method:str,
                        risk_aversion:float,
                        omega_method:str,
                        asset_weight_cons:list=[],
                        port_stats_cons:list=[]):
        """
        Black Litterman 模型             
        - ''returns'' - (pd.DataFrame) columns name is asset name
        - ''asset_list'' - (list) asset name
        - ''market_weights'' - (list) weight of asset [36, 8, 56] i.e. sum up to 100
        - ''investor_views'' - (list of list) investor view information list
        ‘’看好的资产‘’ 比 ‘’看衰的资产‘’ 收益高 ‘’观点内的收益‘’， 信心水平 ''信心水平'' 
            每单元由四部分组成：[[],[],float,float]
                        1. 看好的资产列表 list 
                        2. 看衰的资产列表 list 
                                            如果看衰的资产列表非空，表示观点是相对观点 As 好于 Bs
                                            如果看衰的资产列表为空，表示观点是绝对观点 As 好
                        3. 观点内的收益 float 
                        4. 信心水平    float
        - ''tau'' - (float) investor view parameter less than 0.05,
        - ''opt_method'' - (str), 'max_sharpe' 'max_ret' 'min_std'
        - ''risk_aversion'' - (float) risk aversion, expected measures of returns with one measure of risk
        - ''omega_method'' - (str) 'prior_variance' or 'user_confidences'
                            prior_variance: investor confidence calculated by market history
                            user_confidences: investor confidence provide by user
        - ''asset_weight_cons'' - (list of list) 某资产权重限制 
            每单元由三部分组成： [000001!0, '<=', 0.6]
                        1 资产id （str） 
                        2 大小方式（str） ’=’ ‘>=’ '<='
                        3 权重数  (float)    
        - ''port_stats_cons'' - (list of list) 组合指标限制
            每单元由三部分组成：['std', '<=', 0.2]
                        1 指标：['ret','std','sharpe']
                        2 大小方式（str） ’=’ ‘>=’ '<='
                        3 指标数  (float)       
        """
        investor_views, pick_list, view_confidences = self._view_parse(investor_views,returns,market_weights)
        cons = self._cons_parse(returns, asset_weight_cons, port_stats_cons)

        # step 0 data prepare
        returns.loc[:,'cash'] = 0
        asset_list = returns.columns.tolist()
        market_capitalised_weights = market_weights + [0]
        covariance = returns.cov()
        asset_names = returns.columns.tolist()
        noa = len(asset_list)
        bnds = tuple((0,1) for x in range(noa)) # 权重限制在0,1之间 不允许做空

        # step 1
        num_assets = len(market_capitalised_weights)
        num_views = len(view_confidences)
        if asset_names is None:
            if covariance is not None and isinstance(covariance, pd.DataFrame):
                asset_names = covariance.columns
            else:
                asset_names = list(map(str, range(num_assets)))

        # step 2 pre process inputs
        investor_views = np.array(np.reshape(investor_views, newshape=(len(investor_views), 1)))
        market_capitalised_weights = np.array(np.reshape(market_capitalised_weights, newshape=(len(market_capitalised_weights), 1)))
        if isinstance(covariance, pd.DataFrame):
            covariance = covariance.values

        # step 3 calculate implied equilbrium returns
        implied_equilibrium_returns = risk_aversion * covariance.dot(market_capitalised_weights)

        # step 4 create the pick matrix (P) from user specified assets involved in the views
        pick_matrix = np.zeros((num_views, num_assets))
        pick_matrix = pd.DataFrame(pick_matrix, columns=asset_names)
        for view_index, pick_dict in enumerate(pick_list):
            assets = list(pick_dict.keys()) 
            values = list(pick_dict.values())
            pick_matrix.loc[view_index, assets] = values
        pick_matrix = pick_matrix.values

        # step 5 build the covariance matrix of errors in investor views (omega)
        if omega_method == 'prior_variance':
            print('without view confidences')
            omega = pick_matrix.dot((tau * covariance).dot(pick_matrix.T))
        else:
            print('include detault view confidences ')
            view_confidences = np.array(np.reshape(view_confidences, (1, len(view_confidences))))
            alpha = (1 - view_confidences) / view_confidences
            omega = alpha * pick_matrix.dot(covariance).dot(pick_matrix.T)
        omega = np.diag(np.diag(omega))
        omega = np.array(np.reshape(omega, (num_views, num_views)))

        # step 6 BL expected returns
        posterior_expected_returns = implied_equilibrium_returns + (tau * covariance).dot(pick_matrix.T).\
            dot(inv(pick_matrix.dot(tau * covariance).dot(pick_matrix.T) + omega).dot(investor_views - pick_matrix.dot(implied_equilibrium_returns)))
        posterior_expected_returns = posterior_expected_returns.reshape(1, -1)

        # step 7 BL covariance
        posterior_covariance = covariance + (tau * covariance) - (tau * covariance).dot(pick_matrix.T).\
                    dot(inv(pick_matrix.dot(tau * covariance).dot(pick_matrix.T) + omega)).dot(pick_matrix).dot(tau * covariance)

        if opt_method == 'max_sharpe':
            opt_func = self._max_sharpe
        elif opt_method == 'max_ret':
            opt_func = self._max_ret
        elif opt_method == 'min_std':
            opt_func = self._min_vol
        opts = sco.minimize(fun=opt_func, args=(returns,posterior_expected_returns,posterior_covariance), x0=noa*[1./noa,], method='SLSQP', bounds=bnds, constraints=cons)
        self.weights = self._get_opt_weight(opts, asset_list)
        self.port_stats = self._get_port_stats(opts, returns)

class RiskBudgetModel(ModelCalulator):

    def __init__(self,risk_free:float=0.03, trading_day_per_year:int=242): 
        super().__init__(risk_free=risk_free, trading_day_per_year=trading_day_per_year)

    def process(self, risk_method:str, fund_nav:pd.DataFrame, targets:list=[]):
        """
        风险预算模型
        - ''risk_method'' - (str) risk measure: annual_vol, mdd, var, cvar
        - ''fund_nav'' - (pd.DataFrame) fund nav, columns is asset name
        - ''targets'' - (list) risk budget, if is None, turn into Risk Parity Model
        """
        returns = fund_nav.pct_change(1).iloc[1:]
        if targets == []:
            targets = returns.shape[1] * [1/returns.shape[1]]
        x_0 = returns.shape[1] * [1/returns.shape[1]]
        cons = ({'type': 'eq', 'fun': self._total_weight_constraint},
        {'type': 'ineq', 'fun': self._long_only_constraint})
        res= sco.minimize(fun=self._risk_budget_objective, x0=x_0, args=(risk_method,targets,returns,fund_nav,self.trading_day_per_year), method='SLSQP',constraints=cons, options={'disp': True})
        self.weights = list(res.x)
        port_asset_risk = self._calculate_risk_contribution(self.weights, risk_method, returns, fund_nav, self.trading_day_per_year)
        self.port_asset_risk = list(np.array(port_asset_risk).T[0])

def test_mv():
    # 均值方差模型
    from ..data.api.basic import BasicDataApi
    fund_list = ['000001!0','000006!0','000011!0']
    fund_nav_ori = BasicDataApi().get_fund_nav(fund_list=fund_list)
    fund_nav = fund_nav_ori.pivot_table(index='datetime',columns='fund_id',values='adjusted_net_value').dropna()
    returns = fund_nav.pct_change(1).iloc[1:]
    # 夏普最大 限制资产权重测试
    opt_method = 'max_sharpe'
    asset_weight_cons = [['000006!0','<=',0.6]]
    port_stats_cons = []
    self = MeanVarianceModel()
    self.process(returns, opt_method, asset_weight_cons,port_stats_cons)
    w = self.get_opt_weight()
    p_stats = self.get_ports_stats()
    print(w)
    print(p_stats)

    # 收益最大 限制波动率测试
    opt_method = 'max_ret'
    asset_weight_cons = []
    port_stats_cons = [['std', '<=', 0.2]]
    self = MeanVarianceModel()
    self.process(returns,opt_method,asset_weight_cons,port_stats_cons)
    w = self.get_opt_weight()
    p_stats = self.get_ports_stats()
    print(w)
    print(p_stats)

def test_bl():
    # black litterman 模型
    from ..data.api.basic import BasicDataApi
    fund_list = ['000001!0','000006!0','000011!0']
    fund_nav_ori = BasicDataApi().get_fund_nav(fund_list=fund_list)
    fund_nav = fund_nav_ori.pivot_table(index='datetime',columns='fund_id',values='adjusted_net_value').dropna()
    returns = fund_nav.pct_change(1).iloc[1:]
    # 测试用用户设定的信心水平
    market_weights = [36, 8, 56]
    investor_view = [[['000006!0'],['000011!0'],0.03,0.5]]
    tau = 0.0019
    opt_method = 'max_sharpe'
    risk_aversion = 5
    omega_method='user_confidences'#'prior_variance'
    asset_weight_cons = []
    port_stats_cons = []
    self = BlackLittermanModel()
    self.process(returns,market_weights,investor_view,tau,opt_method,risk_aversion,omega_method,asset_weight_cons,port_stats_cons)
    w = self.get_opt_weight()
    p_stats = self.get_ports_stats()
    print(w)
    print(p_stats)

    # 测试不用用户设定的信心水平
    market_weights = [36, 8, 56]
    investor_view = [[['000006!0'],['000011!0'],0.03,0.5]]
    tau = 0.0019
    opt_method = 'max_sharpe'
    risk_aversion = 5
    omega_method='user_confidences'#'prior_variance'
    asset_weight_cons = []
    port_stats_cons = []
    self = BlackLittermanModel()
    self.process(returns,market_weights,investor_view,tau,opt_method,risk_aversion,omega_method,asset_weight_cons,port_stats_cons)
    w = self.get_opt_weight()
    p_stats = self.get_ports_stats()
    print(w)
    print(p_stats)

def test_rp():
    # 风险平价模型
    from ..data.api.basic import BasicDataApi
    fund_list = ['000001!0','000006!0','000011!0']
    fund_nav_ori = BasicDataApi().get_fund_nav(fund_list=fund_list)
    fund_nav = fund_nav_ori.pivot_table(index='datetime',columns='fund_id',values='adjusted_net_value').dropna()
    risk_method = 'annual_vol'
    self = RiskBudgetModel()
    self.process(risk_method, fund_nav)
    w = self.get_opt_weight()
    asset_risk_contribution = self.get_asset_risk()
    print(w)
    print(asset_risk_contribution)

def test_rb():
    # 风险预算模型
    from ..data.api.basic import BasicDataApi
    fund_list = ['000001!0','000006!0','000011!0']
    fund_nav_ori = BasicDataApi().get_fund_nav(fund_list=fund_list)
    fund_nav = fund_nav_ori.pivot_table(index='datetime',columns='fund_id',values='adjusted_net_value').dropna()
    risk_method = 'annual_vol'
    targets = [0.5,0.25,0.25]
    self = RiskBudgetModel()
    self.process(risk_method, fund_nav, targets)
    w = self.get_opt_weight()
    asset_risk_contribution = self.get_asset_risk()
    print(w)
    print(asset_risk_contribution)


from . import Singleton
import platform
import json

CURRENT_PLATFORM = platform.system()
DEFAULT_CONFIG_PATH = '/shared/etc/ifa/config.json'

class Configurator(metaclass=Singleton):

    def __init__(self, config_path=DEFAULT_CONFIG_PATH):
        with open(config_path) as f:
            self.config = json.load(f)
